;****************************************************************************
;
;	FILE:	Homo.asm
;
;	PURPOSE:	Homophonic substitution crypt/decrypt
;
;	COMMENTS:	Not very flexible with the HomoBox & HomoDecryptBox
;				hardcoded. If you change it, be shure to test it good.
;				It's easy to get bugs if you change the boxes.
;				See: homo.inc
;
;	FUNCTIONS:	HomoCrypt PROTO pClear:DWORD,pChiper:DWORD,len:DWORD
;				GetCryptChar PROTO
;				HomoDecrypt PROTO pClear:DWORD,pChiper:DWORD,len:DWORD
;				GetBoxIndex PROTO
;
;	HISTORY:
;
;	CREATED:	2002-03-25 22:45 Basse
;
;****************************************************************************
.386
.model flat, stdcall
option casemap:none
include \masm32\include\windows.inc
include \masm32\include\kernel32.inc
include \masm32\include\advapi32.inc
include \masm32\include\user32.inc
include \masm32\include\masm32.inc
;include \masm32\include\mylib.inc
include homo.inc
includelib \masm32\lib\user32.lib
includelib \masm32\lib\kernel32.lib
includelib \masm32\lib\advapi32.lib
includelib \masm32\lib\masm32.lib
;includelib \masm32\lib\mylib.lib

HomoCrypt PROTO pClear:DWORD,pChiper:DWORD,len:DWORD
GetCryptChar PROTO
HomoDecrypt PROTO pClear:DWORD,pChiper:DWORD,len:DWORD
GetBoxIndex PROTO

.code
;============================================================================
;
;	FUNCTION:	HomoCrypt
;
;	PURPOSE:	Crypting text with homophonic substitution crypto.
;
;	ARGUMENTS:	pClear:DWORD ptr to the text to crypt
;				pChiper:DWORD ptr to buffer for crypted text
;				len:DWORD length of text
;
;	COMMENTS:
;
;	PROTOTYPE:	HomoCrypt PROTO pClear:DWORDpChiper:DWORDlen:DWORD
;
;	RETURNS:	No
;
;	HISTORY:
;
;	CREATED:	2002-03-25 22:53 Basse
;
;============================================================================
HomoCrypt proc uses ebx ecx edi esi pClear:DWORD,pChiper:DWORD,len:DWORD
	LOCAl hWorkBuf:DWORD

	;Clear counter variables
	lea edi,a_cnt
	mov al,0
	mov ecx,26
	rep stosb

	;Allocate memory for work buffer
	mov eax,len
	inc eax
    invoke LocalAlloc , LMEM_ZEROINIT, eax
    mov hWorkBuf,eax
    invoke MemCopy ,pClear,hWorkBuf,len
	invoke ucase ,hWorkBuf

	mov esi,hWorkBuf
	mov edi,pChiper
	mov ecx,len

	;Crypt the text
cLoop:
	mov al,byte ptr[esi]
	.if al>= "A" && al<="Z" ;Is letter?
		invoke GetCryptChar
	.endif
	mov byte ptr[edi],al
	inc esi
	inc edi
	loop cLoop

	invoke LocalFree ,hWorkBuf
	ret
HomoCrypt endp

;============================================================================
;
;	FUNCTION:	HomoDecrypt
;
;	PURPOSE:	Decrypt text crypted with homophonic substitution crypto.
;
;	ARGUMENTS:	pClear:DWORD ptr to buffer to store clear text in
;				pChiper:DWORD ptr to crypted text
;				len:DWORD length of crypted text
;
;	COMMENTS:
;
;	PROTOTYPE:	HomoDecrypt PROTO pClear:DWORDpChiper:DWORDlen:DWORD
;
;	RETURNS:	No
;
;	HISTORY:
;
;	CREATED:	2002-03-25 22:53 Basse
;
;============================================================================
HomoDecrypt proc uses ecx edi esi pClear:DWORD,pChiper:DWORD,len:DWORD

	mov esi,pChiper
	mov edi,pClear
	mov ecx,len

	;Decrypt
cLoop:
	mov al,byte ptr[esi]
	invoke GetBoxIndex
	.if eax==-1
		jmp save				;Not decrypted char, .!,;+- etc
	.endif
	mov al,byte ptr[HomoDecryptBox+eax]
save:
	mov byte ptr[edi],al
next:
	inc esi
	inc edi
	loop cLoop

	ret
HomoDecrypt endp

;============================================================================
;
;	FUNCTION:	GetBoxIndex
;
;	PURPOSE:	Return the box index of HomoBox for char in al
;
;	ARGUMENTS:	al is char to get box index for
;
;	COMMENTS:
;
;	PROTOTYPE:	GetBoxIndex PROTO
;
;	RETURNS:	Box index in eax
;
;	HISTORY:
;
;	CREATED:	2002-03-25 22:58 Basse
;
;============================================================================
GetBoxIndex proc uses ecx esi

	lea esi,HomoBox

	;Hash a bit...
	.if al<="9"
		jmp @F          ;Search in row 0
	.elseif al<="J"
		add esi,10		;Search in row 1
	.elseif al<="T"
		add esi,20      ; ...
	.elseif al<="a"
		add esi,30
	.elseif al<="k"
		add esi,40
	.elseif al<="u"
		add esi,50
	.elseif al<=0C1h
		add esi,60
	.elseif al<=0CBh
		add esi,70
	.elseif al<=0E5h
		add esi,80
	.elseif al<=0EFh
		add esi,90
	.endif
@@:
	mov ecx,10			;Chars in a row
floop:

	.if al==byte ptr[esi]
		mov eax,esi				;Calculate index
		sub eax,offset HomoBox
	.else
		inc esi
		loop floop
		mov eax,-1              ;No match if we end up here
	.endif
	ret
GetBoxIndex endp

;============================================================================
;
;	FUNCTION:	GetCryptChar
;
;	PURPOSE:	Crypt the char in al with homophonic crypto
;
;	ARGUMENTS:	Char to crypt in al
;
;	COMMENTS:
;
;	PROTOTYPE:	GetCryptChar PROTO
;
;	RETURNS:	Crypted char in al
;
;	HISTORY:
;
;	CREATED:	2002-03-25 23:02 Basse
;
;============================================================================
GetCryptChar proc uses ebx ecx edx edi esi

	lea esi, HomoBox		;Get crypt char here
	mov dl,al
	and eax,0FFh			;mask out the char
	sub al,41h
	lea edi,a               ;ptr to offset ptrs
	shl al,2				;to dwords
	add di,ax ;al           ;get current offset ptr
	mov edi,dword ptr[edi]	;Offsets ptr, ?_off

	lea ebx,a_cnt			;Start of char counters
	shr eax,2				;to bytes
	add ebx,eax         	;ebx = pCurCounter
    xor ecx,ecx
    mov cl,byte ptr[ebx]
	mov al,byte ptr[edi+ecx]		;Offset in HomoBox
	inc byte ptr[ebx] 		;a_cnt

	;12 Test for the most common letters first
	.if dl=="E"
		.if byte ptr[ebx]==12
			jmp overflow
		.endif
	;9 substitute chars
	.elseif dl=="T"
		.if byte ptr[ebx]==9
			jmp overflow
		.endif
	;8 substitute chars
	.elseif dl=="A"
		.if byte ptr[ebx]==8
			jmp overflow
		.endif
	;7 substitute chars
	.elseif dl=="O"
		.if byte ptr[ebx]==7
			jmp overflow
		.endif
	;6 substitute chars
	.elseif dl=="H" || dl=="I"  || dl=="N"  || dl=="R"  || dl=="S"
		.if byte ptr[ebx]==6
			jmp overflow
		.endif
	;4 substitute chars
	.elseif dl=="D" || dl=="L"
		.if byte ptr[ebx]==4
			jmp overflow
		.endif
	;3 substitute chars
	.elseif dl=="C" || dl=="U"
		.if byte ptr[ebx]==3
			jmp overflow
		.endif
	;2 substitute chars
	.elseif dl=="B" || dl=="F" || dl=="G" || dl=="M" || dl=="P" || dl=="W" || dl=="Y"
		.if byte ptr[ebx]==2
			jmp overflow
		.endif
	;1 substitute chars
	.elseif dl=="J" || dl=="K" || dl=="Q" || dl=="V" || dl=="X" || dl=="Z"
		jmp overflow
	.endif
	jmp done
overflow:
	mov byte ptr[ebx],0		;Reset char substitute counter

done:
	and eax,0FFh
    mov al,byte ptr[esi+eax]
exit:
	ret
GetCryptChar endp

end

