;****************************************************************************
;
;	FILE:		Vigenere.asm
;
;	PURPOSE:	Vigenre crypt/decrypt of text a-z
;
;	COMMENTS:
;
;	FUNCTIONS:  VigenereCrypt proc pClear:DWORD,pCrypt:DWORD,pKey:DWORD,len:DWORD
;				VigenereDecrypt proc pClear:DWORD,pCrypt:DWORD,pKey:DWORD,len:DWORD
;
;	HISTORY:
;
;	CREATED:	2002-03-25 22:05 Basse
;
;	DESCRIPTION:
;
;	The Vigenre table
;   -----------------------------
;	   abcdefghijklmnopqrstuvwxyz
;	1  BCDEFGHIJKLMNOPQRSTUVWXYZA
;	2  CDEFGHIJKLMNOPQRSTUVWXYZAB
;	3  DEFGHIJKLMNOPQRSTUVWXYZABC
;	4  EFGHIJKLMNOPQRSTUVWXYZABCD
;	5  FGHIJKLMNOPQRSTUVWXYZABCDE
;	6  GHIJKLMNOPQRSTUVWXYZABCDEF
;	7  HIJKLMNOPQRSTUVWXYZABCDEFG
;	8  IJKLMNOPQRSTUVWXYZABCDEFGH
;	9  JKLMNOPQRSTUVWXYZABCDEFGHI
;	10 KLMNOPQRSTUVWXYZABCDEFGHIJ
;	11 LMNOPQRSTUVWXYZABCDEFGHIJK
;	12 MNOPQRSTUVWXYZABCDEFGHIJKL
;	13 NOPQRSTUVWXYZABCDEFGHIJKLM
;	14 OPQRSTUVWXYZABCDEFGHIJKLMN
;	15 PQRSTUVWXYZABCDEFGHIJKLMNO
;	16 QRSTUVWXYZABCDEFGHIJKLMNOP
;	17 RSTUVWXYZABCDEFGHIJKLMNOPQ
;	18 STUVWXYZABCDEFGHIJKLMNOPQR
;	19 TUVWXYZABCDEFGHIJKLMNOPQRS
;	20 UVWXYZABCDEFGHIJKLMNOPQRST
;	21 VWXYZABCDEFGHIJKLMNOPQRSTU
;	22 WXYZABCDEFGHIJKLMNOPQRSTUV
;	23 XYZABCDEFGHIJKLMNOPQRSTUVW
;	24 YZABCDEFGHIJKLMNOPQRSTUVWX
;	25 ZABCDEFGHIJKLMNOPQRSTUVWXY
;	26 ABCDEFGHIJKLMNOPQRSTUVWXYZ
;
;	KeyWord:      KING
;	Clear text:   the
;	Crypted text: the -> DPR
;
;	Crypt text:
;	(convert clear text to upper)
;	x="A" + "K" - "A" + "T" - "A" =
;	= "K" + "T" - "A"
;	if x > "Z"
; 		sub x,26
;	endif
;
;	Decrypt text:
;	x="D"-"K"
;	if x<0
;  		add x,26
;	endif
;	add x,"A"
;
;****************************************************************************
.386
.model flat, stdcall
option casemap:none
include \masm32\include\windows.inc
include \masm32\include\kernel32.inc
include \masm32\include\advapi32.inc
include \masm32\include\user32.inc
include \masm32\include\masm32.inc
includelib \masm32\lib\user32.lib
includelib \masm32\lib\kernel32.lib
includelib \masm32\lib\advapi32.lib
includelib \masm32\lib\masm32.lib

VigenereCrypt PROTO pClear:DWORD,pCrypt:DWORD,pKey:DWORD,len:DWORD
VigenereDecrypt PROTO pClear:DWORD,pCrypt:DWORD,pKey:DWORD,len:DWORD

.code
;============================================================================
;
;	FUNCTION:	VigenereCrypt
;
;	PURPOSE:	Crypt a text with the Vigenre crypto
;
;	ARGUMENTS:	pClear:DWORD ptr to the text to crypt
;			pCrypt:DWORD ptr to memory for crypted text
;			pKey:DWORD   ptr to the key to use for the crypto.
;			len:DWORD    length of the text
;
;	COMMENTS:
;
;	PROTOTYPE:	VigenereCrypt PROTO pClear:DWORD,pCrypt:DWORD,pKey:DWORD,len:DWORD
;
;	RETURNS:	No
;
;	HISTORY:
;
;	CREATED:	2002-03-25 22:12 Basse
;
;============================================================================
VigenereCrypt proc uses ebx ecx edi esi pClear:DWORD,pCrypt:DWORD,pKey:DWORD,len:DWORD
    LOCAL KeyLen:DWORD
	LOCAL Index:DWORD

	;Get key len
	invoke lnstr ,pKey
	mov KeyLen,eax

	;Clear index (index to pClear)
	mov Index,0

	;Crypt
	mov esi,pClear ;hWorkBuf
	mov edi,pCrypt
	mov ecx,len
cLoop:
	mov al,byte ptr[esi]
	.if al>="a" && al<="z"
		sub al,20h					;To upper
	.endif
	.if al>="A" && al<="Z"
		mov eax,Index
		mov ebx,KeyLen
		xor edx,edx
		div ebx     				;Index mod KeyLen (result in edx)
		mov ebx,pKey ;hKeyBuf
		mov ah,byte ptr[ebx+edx] 	;Key char
		mov al,byte ptr[esi]        ;Clear char

		.if al>="a" && al<="z"
			sub al,20h					;Clear char to upper
		.endif

		.if ah>="a" && ah<="z"
			sub ah,20h				;Key char to upper
		.endif

		;The crypting
		add al,ah
		sub al,"A"+"A"
		cmp al,25
		jng @F
		sub al,26 					;Overflow, wrap around
	@@:
		add al,"A"

		inc Index
	.endif
	mov byte ptr[edi],al
next:
	inc esi
	inc edi
	loop cLoop
	ret
VigenereCrypt endp

;============================================================================
;
;	FUNCTION:	VigenereDecrypt
;
;	PURPOSE:	Decrypt a text crypted with the Vigenre crypto
;
;	ARGUMENTS:	pClear:DWORD ptr to buffer for the decrypted text
;			pCrypt:DWORD ptr to crypted text
;			pKey:DWORD   ptr to the key to use for the decryption.
;			len:DWORD    length of the crypted text
;
;	COMMENTS:
;
;	PROTOTYPE:	VigenereDecrypt PROTO pClear:DWORD,pCrypt:DWORD,pKey:DWORD,len:DWORD
;
;	RETURNS:	No
;
;	HISTORY:
;
;	CREATED:	2002-03-25 22:12 Basse
;
;============================================================================
VigenereDecrypt proc uses ebx ecx edi esi pClear:DWORD,pCrypt:DWORD,pKey:DWORD,len:DWORD
    LOCAL KeyLen:DWORD
	LOCAL Index:DWORD

	;Get key len
	invoke lnstr ,pKey
	mov KeyLen,eax

	;Clear index (index to pClear)
	mov Index,0

	;Decrypt
	mov esi,pCrypt ;pClear
	mov edi,pClear
	mov ecx,len
dLoop:
	mov al,byte ptr[esi]
	.if al>="a" && al<="z"
		sub al,20h					;To upper
	.endif
	.if al>="A" && al<="Z"
		mov eax,Index
		mov ebx,KeyLen
		xor edx,edx
		div ebx     				;Index mod KeyLen (result in edx)
		mov ebx,pKey ;hKeyBuf
		mov ah,byte ptr[ebx+edx] 	;Key char
		mov al,byte ptr[esi]        ;Crypt char

		.if al>="a" && al<="z"
			sub al,20h				;Crypt char to upper
		.endif

		.if ah>="a" && ah<="z"
			sub ah,20h				;Key char to upper
		.endif

		;The decryption
		sub al,ah
		cmp al,0
		jge @F
		add al,26					;Underflow, wrap around.
	@@:
		add al,"A"

		inc Index
	.endif
	mov byte ptr[edi],al
next:
	inc esi
	inc edi
	loop dLoop

	invoke lcase ,pClear			;Convert clear text to lower
	ret
VigenereDecrypt endp

end

